/* AbmlParser.java */
package sams.chp2;

import java.util.*;
import java.io.*;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

public class AbmlParser
{
    public final static boolean debug;

    static
    {
        String strDebug = System.getProperty("DEBUG");
        if (strDebug == null)
            strDebug = System.getProperty("debug");

        if (strDebug != null && strDebug.equalsIgnoreCase("true"))
            debug = true;
        else
            debug = false;
    }

    private Parser saxParser;

    private AbmlHandler docHandler;

    class AbmlHandler implements org.xml.sax.DocumentHandler
    {
        /** locator object from parser. */
        private Locator loc;

        /** Vector of addresses. */
        private Vector addresses;

        /** Current element parsed. */
        private int currentElement;

        /** current Address */
        private Address currentAddress;

        /** accessor method. */
        public final Vector getAddresses() { return addresses; }

        /** method of the DocumentHandler Interface. */
        public void characters(char[] ch, int start, int length)
        {
            // Receive notification of character data inside an element.
            if (debug) System.out.println("Called characters(ch:" +
                               new String(ch,start,length) +
                               ",start:" + start + ",length: " + length + ")");

            if (currentAddress == null)
                return; // parser will catch this well-formedness error.

            String s = new String(ch, start, length);
            switch (currentElement)
            {
                case Address.NAME:
                    currentAddress.setName(s);
                break;
                case Address.STREET:
                    Vector v = currentAddress.getStreets();
                    if (v == null)
                        v = new Vector();
                    v.addElement(s);
                    currentAddress.setStreets(v);
                break;
                case Address.CITY:
                    currentAddress.setCity(s);
                break;
                case Address.STATE:
                    currentAddress.setState(s);
                break;
                case Address.ZIP:
                    currentAddress.setZip(s);
                break;
            }
        }

        /** method of the DocumentHandler Interface. */
        public void startDocument()
        {
            // Receive notification of the beginning of the document.
            if (debug) System.out.println("Called startDocument()");

            // initialive the vector
            addresses = new Vector();
        }

        /** method of the DocumentHandler Interface. */
        public void endDocument()
        {
            // Receive notification of the end of the document.
            if (debug) System.out.println("Called endDocument()");
        }

        /** method of the DocumentHandler Interface. */
        public void startElement(java.lang.String name, AttributeList attributes)
        {
            // Receive notification of the start of an element.
            if (debug) System.out.println("Called startElement(name:" + name + ")");
            if (name.equals("ADDRESS"))
            {
                // create an Address object
                currentElement = Address.ADDRESS;
                currentAddress = new Address();
            }
            else if (name.equals("NAME"))
                currentElement = Address.NAME;
            else if (name.equals("STREET"))
                currentElement = Address.STREET;
            else if (name.equals("CITY"))
                currentElement = Address.CITY;
            else if (name.equals("STATE"))
                currentElement = Address.STATE;
            else if (name.equals("ZIP"))
                currentElement = Address.ZIP;
            else
                currentElement = -1;
        }

        /** method of the DocumentHandler Interface. */
        public void endElement(java.lang.String name)
        {
            // Receive notification of the end of an element.
            if (debug) System.out.println("Called endElement(name: " + name + ")");

            if (name.equals("ADDRESS"))
                addresses.addElement(currentAddress);
        }

        /** method of the DocumentHandler Interface. */
        public void ignorableWhitespace(char[] ch, int start, int length)
        {
            // Receive notification of ignorable whitespace in element content.
            if (debug) System.out.println("Called ignorableWhitespace(ch:"
                                          + new String(ch,start,length) +
                             ",start: " + start + ",length: " + length + ")");
        }

        /** method of the DocumentHandler Interface. */
        public void processingInstruction(java.lang.String target,
                                   java.lang.String data)
        {
            // Receive notification of a processing instruction.
            if (debug) System.out.println("Called processingInstruction(target:"
                                          + target + ",data:" + data + ")");
        }

        /** method of the DocumentHandler Interface. */
        public void setDocumentLocator(Locator locator)
        {
            // Receive a Locator object for document events.
            if (debug) System.out.println("Called setDocumentLocator()");
            loc = locator;
        }
    }

    public AbmlParser() throws InstantiationException
    {
        try
        {
            saxParser = ParserFactory.makeParser();
        } catch (Exception e)
          {
            if (e instanceof InstantiationException)
                throw (InstantiationException) e;
            else
                throw new InstantiationException("Reason:" + e.toString());
          }

        docHandler = new AbmlHandler();
        saxParser.setDocumentHandler(docHandler);
    }

    /**
      * method to parse an abml file and return a Vector of Address objects.
      * @param is  org.xml.sax.InputSource.
      * @returns A Vector of sams.chp2.Address objects.
      */
    public Vector parse(InputSource is) throws SAXException
    {
        try
        {
            saxParser.parse(is);
        } catch (IOException ioe) { throw new SAXException(ioe.getMessage()); }

        return docHandler.getAddresses();
    }

    /** main() method for unit testing. */
    public static void main(String args[])
    {
        if (args.length < 1)
        {
            System.out.println("USAGE: java -Dorg.xml.sax.parser=<classname> " +
			 "sams.chp2.AbmlParser <document>");
            System.exit(1);
        }

        try
        {
            AbmlParser addressParser = new AbmlParser();
            File f = new File(args[0]);
            InputSource is = new InputSource(f.toURL().toString());
            Vector docAddresses = addressParser.parse(is);

            // how many addresses?
            int count = docAddresses.size();
            System.out.println("# of addresses: " + count);

            // print out the address names
            for (int i=0; i < count; i++)
            {
                System.out.println("Address of: " +
                                    ((Address) docAddresses.elementAt(i)).getName());
            }
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}

