/* BookmarkViewer.java */
package sams.chp2;

import com.microstar.xml.*;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class BookmarkViewer extends Applet implements XmlHandler,
                                                      ActionListener,
                                                      ItemListener
{
    private final boolean debug = false;
    private boolean isApplet = true;
    public final String datasource = "bookmarks.xml";
    XmlParser parser = new XmlParser();
    Vector bookmarks = new Vector();
    Hashtable lookup = new Hashtable();

    final int BOOKMARKLIST = 0;
    final int FOLDER = 1;
    final int BOOKMARK = 2;
    final int COMMENT = 3;

    public class Bookmark
    {
        String name;
        String href;
        String comment;
    }

    Bookmark currentBookmark = null;
    int currentElement = -1;
    String lastName, lastHref;

    // gui components
    Label listLabel = new Label("Bookmarks");
    Label commentLabel = new Label("Comment");
    Label hrefLabel = new Label("URL");
    java.awt.List bookmarkList = new java.awt.List(10);
    TextField hrefTF = new TextField(30);
    Button goButton = new Button("GO!");
    TextArea commentTA = new TextArea(8, 60);

    public void init()
    {
        String surl = null;

        if (isApplet)
        {
            URL codebase = getCodeBase();
            surl = codebase.toString() + "/" + datasource;
        }
        else
        {
            // use current directory
            String currentDirectory = System.getProperty("user.dir");
            String fileSep = System.getProperty("file.separator");
            String file = currentDirectory.replace(fileSep.charAt(0), '/') + '/';
            if (file.charAt(0) != '/')
              file = "/" + file;
            try
            {
                URL baseURL = new URL("file", null, file);
                surl = new URL(baseURL,datasource).toString();
            } catch (MalformedURLException mfue) { surl = null; }
        }

        parser.setHandler(this);
        try
        {
            if (surl != null)
                parser.parse(surl, (String) null, (String) null);
        } catch (Exception e)
          {
            e.printStackTrace();
          }


        // here we should have a full Vector
        // set up our GUI
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1.0; gbc.weighty = 1.0;
        gbc.ipadx = 4; gbc.ipady = 4;
        GridBagLayout gbl = new GridBagLayout();
        setLayout(gbl);
        gbc.gridx = 1; gbc.gridy = 1;
        gbc.gridwidth = 5; gbc.gridheight = 1;
        gbc.anchor = GridBagConstraints.WEST;
        gbl.setConstraints(listLabel, gbc);
        add(listLabel);
        gbc.gridx = 1; gbc.gridy = 3;
        gbc.gridwidth = 5; gbc.gridheight = 7;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.anchor = GridBagConstraints.CENTER;
        gbl.setConstraints(bookmarkList, gbc);
        add(bookmarkList);
        gbc.gridx = 7; gbc.gridy = 1;
        gbc.gridwidth = 1; gbc.gridheight = 1;
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.WEST;
        gbl.setConstraints(hrefLabel, gbc);
        add(hrefLabel);
        gbc.gridx = 8; gbc.gridy = 1;
        gbc.gridwidth = 4; gbc.gridheight = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbl.setConstraints(hrefTF, gbc);
        add(hrefTF);
        gbc.gridx = 13; gbc.gridy = 1;
        gbc.gridwidth = 1; gbc.gridheight = 1;
        gbl.setConstraints(goButton, gbc);
        add(goButton);
        goButton.addActionListener(this);
        gbc.gridx = 7; gbc.gridy = 3;
        gbc.gridwidth = 4; gbc.gridheight = 1;
        gbc.anchor = GridBagConstraints.CENTER;
        gbl.setConstraints(commentLabel, gbc);
        add(commentLabel);
        gbc.gridx = 7; gbc.gridy = 4;
        gbc.gridwidth = 8; gbc.gridheight = 6;
        gbc.fill = GridBagConstraints.BOTH;
        gbl.setConstraints(commentTA, gbc);
        add(commentTA);

        // fill the bookmark list
        bookmarkList.addItemListener(this);
        int count = bookmarks.size();
        if (count > 0)
        {
            for (int i=0; i < count; i++)
            {
                Bookmark b = (Bookmark) bookmarks.elementAt(i);
                /* addItem() replaced by add() in JDK1.2.
                   However, most browsers are still on 1.1. */
                if (b != null && b.name != null)
                {
                    bookmarkList.addItem(b.name);
                    // use the name as the key
                    lookup.put(b.name, b);
                }
            }
        }
    }

    public void actionPerformed(ActionEvent evt)
    {
        String cmd = evt.getActionCommand();

        if (isApplet)
        {
            if (cmd.equals("GO!"))
            {
                try
                {
                    AppletContext context = getAppletContext();
                    String surl = hrefTF.getText();
                    URL u = new URL(surl);
                    context.showDocument(u);
                } catch (MalformedURLException mue)
                  { showStatus("Incorrect URL."); }
            }
        }
        else
            System.out.println("Operation only supported in an Applet.");
    }

    public void itemStateChanged(ItemEvent evt)
    {
        if (evt.getStateChange() == ItemEvent.SELECTED)
        {
            // We can cheat here, we know the List is the source
            String key = bookmarkList.getSelectedItem();
            // Now get the object
            Bookmark b = (Bookmark) lookup.get(key);
            if (b != null)
            {
                hrefTF.setText(b.href);
                commentTA.setText(b.comment);
            }
        }
    }

    public static void main(String args[])
    {
        BookmarkViewer viewer = new BookmarkViewer();
        viewer.isApplet = false;

        // add panel to Frame
        Frame f = new Frame("Test Bookmark Viewer Frame");

        viewer.init();
        f.add("Center", viewer);

        f.addWindowListener(new WindowAdapter()
                            {
                                public void windowClosing(WindowEvent we)
                                { System.exit(0); }
                            });

        // show Frame
        f.setLocation(100,100);
        //f.setSize(500,400);
        f.pack();
        f.setVisible(true);
    }

    private void setElementType(String ename)
    {
        if (ename.equals("BOOKMARKLIST"))
            currentElement = BOOKMARKLIST;
        else if (ename.equals("FOLDER"))
            currentElement = FOLDER;
        else if (ename.equals("BOOKMARK"))
            currentElement = BOOKMARK;
        else if (ename.equals("COMMENT"))
            currentElement = COMMENT;
    }

    /** XmlHandler: Start the document */
    public void startDocument () throws java.lang.Exception
    {
        if (debug) System.out.println("startDocument()");
    }


    /**
    * XmlHandler: End the document.
    */
    public void endDocument () throws java.lang.Exception
    {
        if (debug) System.out.println("endDocument()");

    }


    /**
    * XmlHandler: Resolve an External Entity.
    */
    public Object resolveEntity (String publicId, String systemId)
    throws java.lang.Exception
    {
        if (debug) System.out.println("resolveEntity(publicId: " + publicId +
                                      ",systemId: " + systemId + ")");
        return null;
    }

    /**
    * XmlHandler: Begin an external entity.
    */
    public void startExternalEntity (String systemId)
    throws java.lang.Exception
    {
        if (debug) System.out.println("startExternalEntity(systeId: " + systemId + ")");

    }

    /**
    * XmlHandler: End an external entity.
    */
    public void endExternalEntity (String systemId)
    throws java.lang.Exception
    {
        if (debug) System.out.println("endExternalEntity(systemId: " + systemId + ")");

    }


    /**
    * XmlHandler: Document type declaration.
    */
    public void doctypeDecl (String name, String publicId, String systemId)
    throws java.lang.Exception
    {
        if (debug) System.out.println("doctypeDecl(name: " + name +
                                      ",publicId: " + publicId +
                                      ",systemId: " + systemId + ")");

    }

    /**
    * XmlHandler: Attribute.
    */
    public void attribute (String aname, String value, boolean isSpecified)
    throws java.lang.Exception
    {
        if (debug) System.out.println("attribute(aname: " + aname +
                                       ",value: " + value +
                                       ",isSpecified: " + isSpecified + ")");

        // Aelfred attribute events are BEFORE the startElement call
        if (aname.equals("name"))
            lastName = value;
        else if (aname.equals("href"))
            lastHref = value;
    }

    /**
    * XmlHandler: Start an element.
    */
    public void startElement (String elname)
    throws java.lang.Exception
    {
        if (debug) System.out.println("startElement(elname: " + elname + ")");
        setElementType(elname);
        if (elname.equals("BOOKMARK"))
        {
            currentBookmark = new Bookmark();
            // NOTE: attribute events are before the startElement event.
            //       NON-INTUITIVE!  SAX uses a better way.
            if (lastName != null)
            {
                currentBookmark.name = lastName;
                lastName = null;
            }

            if (lastHref != null)
            {
                currentBookmark.href = lastHref;
                lastHref = null;
            }
        }
    }

    /**
    * XmlHandler: End an element.
    */
    public void endElement (String elname)
    throws java.lang.Exception
    {
        if (debug) System.out.println("endElement(elname: " + elname +")");
        if (elname.equals("BOOKMARK") && currentBookmark != null)
            bookmarks.addElement(currentBookmark);
    }


    /**
    * XmlHandler: Character data. */
    public void charData (char ch[], int start, int length)
    throws java.lang.Exception
    {
        if (debug) System.out.println("charData(ch:" +
                                      new String(ch, start, length) + ")");
        switch (currentElement)
        {
            case BOOKMARKLIST:
                break;
            case FOLDER:
                break;
            case BOOKMARK:
                break;
            case COMMENT:
                if (currentBookmark != null)
                    currentBookmark.comment = new String(ch, start, length);
                break;
        }
    }


    /** XmlHandler: Ignorable whitespace. */
    public void ignorableWhitespace (char ch[], int start, int length)
    throws java.lang.Exception
    {
        if (debug) System.out.println("ignorableWhitespace(...)");

    }

    /** XmlHandler: Processing instruction.*/
    public void processingInstruction (String target, String data)
    throws java.lang.Exception
    {
        if (debug) System.out.println("processingInstruction(target: " + target +
                                      ",data: " + data + ")");

    }


    /** XmlHandler: Fatal XML parsing error. */
    public void error (String message, String systemId, int line, int column)
    throws java.lang.Exception
    {
        if (debug) System.out.println("error(message:" + message + ")");
        if (!isApplet)
            System.exit(1);
    }
}
