/** SaxTester.java */
package sams.chp2;

import java.io.*;

import java.net.*;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

public class SaxTester
{

    public static void main(String args[])
    {
        if (args.length < 1)
        {
            System.out.println("USAGE: java -Dorg.xml.sax.parser=<classname> " +
			 "sam.chps.SaxTester <document>");
            System.exit(1);
        }

        try
        {
            File f = new File(args[0]);

            // create a SAX input source
            InputSource is = new InputSource(f.toURL().toString());

            // instantiate a SAX driver
            Parser sax = ParserFactory.makeParser();

            // create our test handler
            TestHandler handler = new TestHandler();

            // register a document handler
            sax.setDocumentHandler(handler);

            // register the DTD handler
            sax.setDTDHandler(handler);

            // register the entity resolver
            sax.setEntityResolver(handler);

            // register the error handler
            sax.setErrorHandler(handler);

            // start the parsing!
            sax.parse(is);
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}

class TestHandler extends HandlerBase
{
    /** Locator reference. */
    Locator loc;

    /** method of the DocumentHandler Interface. */
    public void characters(char[] ch, int start, int length)
    {
        // Receive notification of character data inside an element.
        System.out.println("Called characters(ch:" + new String(ch,start,length) +
                           ",start:" + start + ",length: " + length + ")");
    }

    /** method of the DocumentHandler Interface. */
    public void endDocument()
    {
        // Receive notification of the end of the document.
        System.out.println("Called endDocument()");
    }

    /** method of the DocumentHandler Interface. */
    public void endElement(java.lang.String name)
    {
        // Receive notification of the end of an element.
        System.out.println("Called endElement(name: " + name + ")");
    }

    /** method of the DocumentHandler Interface. */
    public void ignorableWhitespace(char[] ch, int start, int length)
    {
        // Receive notification of ignorable whitespace in element content.
        System.out.println("Called ignorableWhitespace(ch:" + new String(ch,start,length) +
                                                       ",start: " + start + ",length: " + length + ")");
    }

    /** method of the DocumentHandler Interface. */
    public void processingInstruction(java.lang.String target,
                               java.lang.String data)
    {
        // Receive notification of a processing instruction.
        System.out.println("Called processingInstruction(target:" + target +
                                                         ",data:" + data + ")");
    }

    /** method of the DocumentHandler Interface. */
    public void setDocumentLocator(Locator locator)
    {
        // Receive a Locator object for document events.
        System.out.println("Called setDocumentLocator()");
        loc = locator;
    }

    /** method of the DocumentHandler Interface. */
    public void startDocument()
    {
        // Receive notification of the beginning of the document.
        System.out.println("Called startDocument()");
    }

    /** method of the DocumentHandler Interface. */
    public void startElement(java.lang.String name, AttributeList attributes)
    {
        // Receive notification of the start of an element.
        System.out.println("Called startElement(name:" + name + ")");
        for (int i = 0; i < attributes.getLength(); i++)
        {
            String attName = attributes.getName(i);
            String type = attributes.getType(i);
            String value = attributes.getValue(i);
            System.out.println("att-name:" + attName + ",att-type:" + type + ",att-value:" + value);
        }
        if (loc != null)
            System.out.println("In " + loc.getSystemId() + ",at line " +
                               loc.getLineNumber() + " and col " +
                               loc.getColumnNumber());
    }

    /** method of the DTDHandler Interface. */
    public void unparsedEntityDecl(java.lang.String name,
                            java.lang.String publicId, java.lang.String systemId,
                            java.lang.String notationName)
    {
        // Receive notification of an unparsed entity declaration.
        System.out.println("Called unparsedEntityDecl");
        System.out.println(name);
        System.out.println(publicId);
        System.out.println(systemId);
        System.out.println(notationName);
    }

    /** method of the DTDHandler Interface. */
    public void notationDecl(java.lang.String name,
                      java.lang.String publicId, java.lang.String systemId)
    {
        // Receive notification of a notation declaration.
        System.out.println("Called notationDecl(name: " + name +
                                               ",publicId: " + publicId +
                                               ",systemId: " + systemId + ")");
    }

    /** method of the EntityResolver Interface. */
    public InputSource resolveEntity(java.lang.String publicId,
                                     java.lang.String systemId)
    {
        // Resolve an external entity.
        System.out.println("Called resolveEntity(publicId:" + publicId +
                                                ",systemId:" + systemId + ")");
        return new InputSource(new StringReader("Resolved Entity"));
    }

    /** method of the ErrorHandler Interface. */
    public void error(SAXParseException e)
    {
        // Receive notification of a recoverable parser error.
        System.out.println("Called error(e:" + e + ")");
        if (loc != null)
            System.out.println("In " + loc.getSystemId() + ",at line " +
                               loc.getLineNumber() + " and col " +
                               loc.getColumnNumber());
        e.printStackTrace();
    }

    /** method of the ErrorHandler Interface. */
    public void fatalError(SAXParseException e)
    {
        // Report a fatal XML parsing error.
        System.out.println("Called fatalError(e:" + e + ")");
        if (loc != null)
            System.out.println("In " + loc.getSystemId() + ",at line " +
                               loc.getLineNumber() + " and col " +
                               loc.getColumnNumber());
        e.printStackTrace();
    }

    /** method of the ErrorHandler Interface. */
    public void warning(SAXParseException e)
    {
        // Receive notification of a parser warning.
        System.out.println("Called warning()");
        if (loc != null)
            System.out.println("In " + loc.getSystemId() + ",at line " +
                               loc.getLineNumber() + " and col " +
                               loc.getColumnNumber());
        e.printStackTrace();
    }
}