/* SimpleXmlScanner.java */
package sams.chp2;

import java.io.*;

public class SimpleXmlScanner
{
    public static final int TT_PI = 0;
    public static final int TT_TAG = 1;
    public static final int TT_CONTENT = 2;
    public static final int TT_ENTITY = 3;

    public static final String [] constants = { "PI",
                                                "TAG",
                                                "CONTENT",
                                                "ENTITY" };

    int ttype;
    PushbackReader rdr;
    private int c = 0;
    private char lastChar = 0;

    public final int getType() { return ttype; }

    public String nextToken() throws IOException
    {
        StringBuffer tok = new StringBuffer();
        ttype = -1; // initial state

        outer:
        while ((c = rdr.read()) != -1)
        {
            switch (c)
            {
                case '<':
                    if (tok.length() > 0)
                    {
                        rdr.unread(c);
                        break outer;
                    }
                    ttype = TT_TAG;
                    break;
                case '>':
                    break outer;
                case '?':
                    if (lastChar == '<')
                        ttype = TT_PI;
                    break;
                case '&':
                    ttype = TT_ENTITY;
                    break;
                case ';':
                    break outer;
                default:
                    if (ttype < 0)
                        ttype = TT_CONTENT;
                    tok.append((char) c);
            }
            lastChar = (char) c;
        }

        if (tok.length() < 1)
            return null;

        return tok.toString();
    }

    public SimpleXmlScanner(InputStream is)
    {
        InputStreamReader isr = new InputStreamReader(is);
        rdr = new PushbackReader(isr);
    }

    public static void main(String args[])
    {
        if (args.length < 1)
        {
            System.out.println("USAGE: java sams.chp2.SimpleXmlScanner xmlfile");
            System.exit(1);
        }

        try
        {
            FileInputStream fis = new FileInputStream(args[0]);
            SimpleXmlScanner scanner = new SimpleXmlScanner(fis);

            String tok = null;

            while ( (tok = scanner.nextToken()) != null)
            {
                System.out.println(SimpleXmlScanner.constants[scanner.getType()] +
                                   ":{" + tok + "}");
            }

            fis.close();
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}
