/*
 * @(#)CreateEntry.java        1.0 99/05/28
 *
 * Copyright (c) 1999 Macmillan Computer Publishing. All Rights Reserved.
 *
 */
package sams.chp3;

import java.io.*;
import com.sun.xml.tree.*;
import org.w3c.dom.*;

public class CreateEntry
{
    public static void main (String argv [])
    {
        FileInputStream inStream;
        Document document;
        String xmlDocumentPath = "catalog.xml";
        try
        {
            inStream = new FileInputStream(xmlDocumentPath);
            document = XmlDocument.createXmlDocument(inStream,true);

            NodeList entries = document.getElementsByTagName( "entry" );
             System.out.println("Original catalog contains " + entries.getLength() + " entries");


            // Create a new entry element.
            Element newEntry = document.createElement("entry");
            // Since entry's contain titles we need a title entry
            Element newTitle = document.createElement("title");
            newTitle.appendChild(document.createTextNode("A new book"));
            newEntry.appendChild(newTitle);

            // Likewise add the author and publisher items
            Element newAuthor = document.createElement("author");
            newAuthor.appendChild(document.createTextNode("I.Also Write"));
            newEntry.appendChild(newAuthor);
            Element newPublisher = document.createElement("publisher");
            newPublisher.appendChild(document.createTextNode("We Publishem, Inc."));
            newEntry.appendChild(newPublisher);

            // We also need a price, which contains two attributes, currency and discount
            Element newPrice = document.createElement("price");
            newPrice.appendChild(document.createTextNode("5.95"));

            // We can build attributes either of two ways.
            // by building it up from scratch and inserting it.
            Attr currAttr= document.createAttribute("cur");
            currAttr.setValue("frc"); // this book prices in french francs
            newPrice.setAttributeNode(currAttr);

            // insert it whole
            newPrice.setAttribute("discount","other");
            // or by inserting it whole
            newEntry.appendChild(newPrice);

            // note that the toString method has no defined behavior for DOM objects!
            System.out.println("newEntry = " + newEntry.toString());

            // Now insert it into the document
            Element root = document.getDocumentElement();
            root.appendChild(newEntry);

            //
            //  Now list the entries
            //
            entries = document.getElementsByTagName( "entry" );
            if ( entries.getLength()== 0)
            {
                System.out.println("No entries in this catalog, exiting");
                System.exit(0);
            }

            for (int i = 0; i < entries.getLength(); i++)
            {
                System.out.print(i + " ");
                Element entryNode = (Element) entries.item(i);
                NodeList titles = entryNode.getElementsByTagName ("title");
                if ( titles.getLength() == 0)
                    System.out.println("Unknown");
                else
                {
                    Node titleElement = titles.item(0);
                    //System.out.println("title.getNodeType() returned " +  titleNode.getNodeType());
                    NodeList titleValues = titleElement.getChildNodes();

                    Node titleValue = titleValues.item(0);
                    System.out.print (" "+titleValue.getNodeValue());
                }

                //
                // Now show author entries
                //
                NodeList authors = entryNode.getElementsByTagName ("author");
                if ( authors.getLength() == 0)
                    System.out.println("Unknown");
                else
                {
                    for ( int k = 0; k < authors.getLength(); k++)
                    {
                        Node authorElement = authors.item(k);
                        NodeList authorValues = authorElement.getChildNodes();
                        Node authorValue = authorValues.item(0);
                        System.out.print (" "+authorValue.getNodeValue());
                    }
                }
                //
                // Now add prices
                //

                NodeList prices = entryNode.getElementsByTagName ("price");
                if ( prices.getLength() == 0)
                    System.out.println("Unknown");
                else
                {
                    for ( int k = 0; k < prices.getLength(); k++)
                    {
                        Node priceElement = prices.item(k);
                        NodeList priceValues = priceElement.getChildNodes();
                        Node priceValue = priceValues.item(0);
                        System.out.print (" "+priceValue.getNodeValue());
                        //
                        // We also know that prices have attributes.  Lets get them as well
                        //
                        NamedNodeMap attributes = priceElement.getAttributes();
                        for ( int l = 0; l < attributes.getLength(); l++)
                        {
                            Attr attributeValue = (Attr)attributes.item(l);
                            System.out.print (" "+attributeValue.getName()+":"+attributeValue.getValue());
                        }
                    }
                }
                System.out.println("");
            }
        }
        catch (Exception e)
        {
            System.out.println("Unexpected exception reading document!" +e);
            System.exit (0);
        }


    }
}
