package sams.chp6;

import org.w3c.dom.*;
import java.util.Comparator;

public class DOMCollectionUtilities
{
    static class StringElementComparator implements Comparator
    {
        public int compare (Object e1, Object e2)
        {
            if ( !( e1 instanceof Element)
                 || !( e2 instanceof Element)
                )
                return 0;
            Element element1 = (Element) e1;
            Element element2 = (Element) e2;
            String content1 = element1.getNodeName();
            String content2 = element2.getNodeName();
            if (element1.getNodeType() == org.w3c.dom.Node.TEXT_NODE &&
                element2.getNodeType() == org.w3c.dom.Node.TEXT_NODE)
            {
                content1 = element1.getNodeValue();
                content2 = element2.getNodeValue();
            }
            return (content1.compareTo(content2));
        }
    }

    static class IntegerElementComparator implements Comparator
    {
        public int compare (Object e1, Object e2)
        {
            if ( !( e1 instanceof Element)
                 || !( e2 instanceof Element)
                )
                return 0;
            Element element1 = (Element) e1;
            Element element2 = (Element) e2;

            // we can only compare convert text elements to integers
            if (element1.getNodeType() != org.w3c.dom.Node.TEXT_NODE ||
                element2.getNodeType() != org.w3c.dom.Node.TEXT_NODE)
                return 0;

            String content1 = element1.getNodeValue();
            String content2 = element2.getNodeValue();

            return new Integer(content1).compareTo(new Integer(content2));
            }
    }

    static class FloatElementComparator implements Comparator
    {
        public int compare (Object e1, Object e2)
        {
            if ( !( e1 instanceof Element)
                 || !( e2 instanceof Element)
                )
                return 0;
            Element element1 = (Element) e1;
            Element element2 = (Element) e2;

            // we can only compare convert text elements to float
            if (element1.getNodeType() != org.w3c.dom.Node.TEXT_NODE ||
                element2.getNodeType() != org.w3c.dom.Node.TEXT_NODE)
                return 0;

            String content1 = element1.getNodeValue();
            String content2 = element2.getNodeValue();

            return new Float(content1).compareTo(new Float(content2));
            }
    }
}