package sams.chp6;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;

public class DOMList extends AbstractList
                     implements Cloneable, Serializable
{

    private transient Document parent;
    private transient Object elementData[];
    private int size;

    private void inRangeCheck(int pos)
    {
    if (pos >= size || pos < 0)
        throw new IndexOutOfBoundsException(
           "Position: "+pos+", DOMList size: "+size);
    }

    public DOMList (Document parent,String tag)
    {
        super();
        this.parent = parent;
        NodeList elements = parent.getElementsByTagName( tag );
        if ( elements.getLength()== 0)
        {
            size=0;
            return;
        }
        elementData= new Object[elements.getLength()];
        for (int i = 0; i < elements.getLength(); i++)
        {
            Element anElement = (Element) elements.item(i);
            elementData[i] = anElement;

        } // end for
    }
    public int size()
    {
        return size;
    }
    public boolean isEmpty()
    {
        return size == 0;
    }
      public void add(int index, Object element)
       {
        throw new UnsupportedOperationException( "DOMList does not support add(int,element) operation" );
    }
      public boolean add(Object element)
       {
        throw new UnsupportedOperationException( "DOMList does not support add(element) operation" );
    }
    public Object get(int position)
    {
        inRangeCheck(position);
        return elementData[position];
    }
    public Object set(int position, Object element)
    {
        inRangeCheck(position);
        throw new UnsupportedOperationException( "DOMList does not support add(int,element) operation" );
    }
    public Object remove(int position)
    {
        inRangeCheck(position);
        throw new UnsupportedOperationException( "DOMList does not support remove(int) operation" );
    }
    public int indexOf(Object elem)
    {
        if (elem == null)
            return -1;
        else
        {
            for (int i = 0; i < size; i++)
            if (elem.equals(elementData[i]))
                return i;
        }
        return -1;
    }
    public int lastIndexOf(Object elem)
    {
        if (elem == null)
            return -1;
        else
        {
            for (int i = size-1; i >= 0; i--)
                if (elem.equals(elementData[i]))
                    return i;
        }
        return -1;
    }

    public void clear()
    {
        throw new UnsupportedOperationException( "DOMList does not support clear operation" );
    }
    public boolean addAll(int index, Collection c)
    {
        throw new UnsupportedOperationException( "DOMList does not support addAll(int, Collection) operation" );
    }

    public Iterator iterator()
    {
        return new DOMIterator();
    }
    public ListIterator listIterator()
    {
        return new DOMListIterator();
    }
	private class DOMListIterator extends DOMIterator implements ListIterator
	{
		public boolean hasPrevious()
		{
			if ( length == 0)
                throw new NoSuchElementException();
			 if (pos > 0 && pos <= length)
			 	return true;
			else
				return false;
		}
		public int nextIndex()
		{
			if ( pos == length)
				return pos; // at end
			else
				return pos;

		}
		public int previousIndex()
		{
			if ( pos == 0)
				return -1;
			else
				return pos -1;
		};
        public Object previous()
        {
            if ( length == 0 || pos -1 < 0)
                throw new NoSuchElementException();
            pos--;
            return table[pos+1];

        }
		public void add(Object o) { throw new UnsupportedOperationException(); };
		public void set(Object o)
		{
			table[pos-1] = o;
		}

	}
    private class DOMIterator implements Iterator
    {
        Object[] table = DOMList.this.elementData;
        int length = elementData.length;
        int pos = 0;
        DOMIterator() {};

        public boolean hasNext()
        {
            if (pos < length)
                return true;
            return false;
        }

        public Object next()
        {
            if (pos >= length)
                throw new NoSuchElementException();
            pos++;
            return table[pos-1];

        }

        public void remove()
        {
            throw new UnsupportedOperationException( "DOMIterator does not support remove() operation" );
        }
    }

	public Object[] toArray()
	{
		System.out.println(" Entered DOMList.toArray");
		Object[] array = new Object[elementData.length];
		for ( int i =0; i < elementData.length; i++)
		{
			array[i] = elementData[i];
		}
		System.out.println(" Entered DOMList.toArray");
		return array;
	}

    public static void main (String argv [])
    {
        if (argv.length != 2)
        {

            System.err.println("Usage: java sams.chp8.DOMArray filename tag");
            System.exit(1);
        }

        Document document;
        try
        {
            document = XMLUtil.openDocument(argv[0]);
            System.out.println("Document successfully created!");

            DOMList test = new DOMList(document, argv[1]);
            Iterator it = test.iterator();
            while (it.hasNext())
            {
                Element e = (Element) it.next();
                XMLUtil.displayNodeInfo("",e);
            }

        }
        catch (Exception e)
        {
            System.out.println("Unexpected exception reading document!" +e);
            System.exit (0);
        }
        System.exit (0);

    }

}
