/*
 * @(#)SortExample.java        1.0 99/11/11
 *
 * Copyright (c) 1999 Macmillan Computer Publishing. All Rights Reserved.
 *
 */
package sams.chp6;

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.*;

public class SortExample
{
    ArrayList bookList = new ArrayList();
    public SortExample (String sTitle)
    {
        System.out.println("Creating array of books");
        bookList.add(new Book("Better Living Thru Chemistry", "A. Publisher", 4.27));
        bookList.add(new Book("Java 2 for C/C++ Programmers", "J.Wiley & Sons", 62.50));
        bookList.add(new Book("The Microsoft Visual J++ 1.1 Sourcebook", "J.Wiley & Sons", 39.95));
        bookList.add(new Book("XML and Java","Macmillian Computer Publishers", 49.95));
        bookList.add(new Book("Inside Visual C++ 5.0", "Microsoft Press", 99.99));

        sortList("(Default)");
        boolean done = false;
        InputStreamReader isr = new InputStreamReader(System.in);
        BufferedReader br = new BufferedReader(isr);
        while ( !done)
        {
            System.out.println("Sort by:");
            System.out.println(" 1 for Title");
            System.out.println(" 2 for Publisher");
            System.out.println(" 3 for Price");
            System.out.println(" 4 to add new book");
            System.out.println(" 5 to exit");
            System.out.println("Enter (1,2,3,4,5): ");
            System.out.flush();
            String command;
            try
            {
                command = br.readLine();
                if (command.equals("done"))
                {
                    done=true;
                    continue;
                }
                if (command != null && command.length() > 0)
                {
                    switch (Integer.parseInt(command))
                    {
                        default:
                            System.out.println("Invalid command '"+ command + "'");
                            continue;
                        case 1: // title
                            System.out.println("Sorting by title");
                            sortList("Title");
                            break;
                        case 2: // publisher
                            System.out.println("Sorting by publisher");
                            sortList("Publisher");
                            break;
                        case 3: // price
                            System.out.println("Sorting by price");
                            sortList("Price");
                            break;
                        case 4: // add new book
                            System.out.println("Enter new book information");
                            bookList.add(new Book());
                            break;
                        case 5: // done
                            done = true;
                            continue;
                    } // end switch
                } // end if command
            } catch (IOException e) { done = true;}
        } // end while not done

    }

    private void sortList (String sSortCriteria)
    {
        if (sSortCriteria == null || sSortCriteria.equals(""))
            return;

        Comparator c = null;
        if (sSortCriteria.equals("Title"))
            c = new TitleComparator();
        else if (sSortCriteria.equals("Publisher"))
            c = new PublisherComparator();
        else if (sSortCriteria.equals("Price"))
            c = new CostComparator();
        if ( null == c)
            Collections.sort(bookList);
        else
            Collections.sort(bookList,c);

        displayList(bookList);
    }

    private void displayList(ArrayList al)
    {
        Iterator it = bookList.iterator();
        while (it.hasNext())
        {
            Book p = (Book) it.next();
            System.out.println(p.sTitle + ", " + p.sPublisher+ ", " + p.dPrice);
        }
    }
    public static void main (String args[])
    {
        new SortExample("Sort Example");
    }

    class Book implements Comparable
    {
        String sTitle = "";
        String sPublisher = "";
        double dPrice = 0.0;

        public Book ()
        {
            try
            {
                InputStreamReader isr = new InputStreamReader(System.in);
                BufferedReader br = new BufferedReader(isr);
                System.out.print("Enter title    : ");
                System.out.flush();
                sTitle = br.readLine();
                System.out.print("Enter publisher: ");
                System.out.flush();
                sPublisher = br.readLine();
                System.out.print("Enter price: ");
                System.out.flush();
                String temp = br.readLine();
                dPrice = Double.parseDouble(temp);
            } catch (IOException e) {
                sTitle = sPublisher = "";
                dPrice=0;
                return;
            }
        }
        public String toString()
        {
            String result= "Title:" + sTitle +"Publisher:"+sPublisher + "Price:" + dPrice;
            return result;
        }
        public Book (String sTitle, String sPublisher, double dPrice)
        {
            this.sTitle = sTitle;
            this.sPublisher = sPublisher;
            this.dPrice = dPrice;
        }

        private Comparator defaultComparator = new TitleComparator();
        public int compareTo (Object o)
        {
            return (defaultComparator.compare(this, o));
        }
    }


    class TitleComparator implements Comparator
    {
        public int compare (Object book1, Object book2)
        {
            String sTitle1 = ((Book)book1).sTitle;
            String sTitle2 = ((Book)book2).sTitle;
            return (sTitle1.compareTo(sTitle2));
        }
    }

    class PublisherComparator implements Comparator
    {
        public int compare (Object book1, Object book2)
        {
            String sPublisher1 = ((Book)book1).sPublisher;
            String sPublisher2 = ((Book)book2).sPublisher;
            return (sPublisher1.compareTo(sPublisher2));
        }
    }
    class CostComparator implements Comparator
    {
        public int compare (Object book1, Object book2)
        {
            double price1 = ((Book)book1).dPrice;
            double price2 = ((Book)book2).dPrice;
            if ( price1 == price2 )
                return 0;
            else if (price1 < price2)
                return -1;
            else
                return 1;
        }
    }
}