package sams.chp6;

import java.io.*;
import java.net.*;
import org.w3c.dom.*;
import com.sun.xml.tree.*;

public class XMLUtil
{
    // map the type to a string
    static String mapNodeTypeToString(short nodeType)
    {
        switch(nodeType)
        {
            case org.w3c.dom.Node.ATTRIBUTE_NODE: return "ATTRIBUTE_NODE";
            case org.w3c.dom.Node.CDATA_SECTION_NODE: return "CDATA_SECTION_NODE";
            case org.w3c.dom.Node.COMMENT_NODE: return "COMMENT_NODE";
            case org.w3c.dom.Node.DOCUMENT_FRAGMENT_NODE: return "DOCUMENT_FRAGMENT_NODE";
            case org.w3c.dom.Node.DOCUMENT_NODE: return "DOCUMENT_NODE";
            case org.w3c.dom.Node.DOCUMENT_TYPE_NODE: return "DOCUMENT_TYPE_NODE";
            case org.w3c.dom.Node.ELEMENT_NODE: return "ELEMENT_NODE";
            case org.w3c.dom.Node.ENTITY_NODE: return "ENTITY_NODE";
            case org.w3c.dom.Node.ENTITY_REFERENCE_NODE: return "ENTITY_REFERENCE_NODE";
            case org.w3c.dom.Node.NOTATION_NODE: return "NOTATION_NODE";
            case org.w3c.dom.Node.PROCESSING_INSTRUCTION_NODE: return "PROCESSING_INSTRUCTION_NODE";
            case org.w3c.dom.Node.TEXT_NODE: return "TEXT_NODE";

        }
        return "Unknown";
    }

    // Display attribute information
    static void displayAttributeInfo(String prefix, Node node)
    {
        // only elements have attributes
        if (node.getNodeType() != org.w3c.dom.Node.ELEMENT_NODE) return;

        NamedNodeMap attributes = node.getAttributes();
        if ( null == attributes || attributes.getLength() == 0)
        {
            return;
        }

        System.out.println(prefix +"has " + attributes.getLength() +" attributes");
        System.out.println(prefix + attributes.toString());
        for (int i = 0; i < attributes.getLength(); i++)
        {
            Node attribute = attributes.item(i);
            System.out.print(prefix+"["+i+"] " + attribute.getNodeName());
            System.out.println(" = " + attribute.getNodeValue());
        }

    }

    // Display generalized node properties
    static void displayNodeInfo(String prefix,Node node)
    {
        System.out.println(prefix+ "----------------");
        System.out.println(prefix + "name:"+node.getNodeName());
        System.out.println(prefix + "type:("+node.getNodeType()+ ")" +mapNodeTypeToString(node.getNodeType()));
        System.out.println(prefix + "value:"+ node.getNodeValue());
        displayAttributeInfo(prefix,node);
        if (node.getNodeType() != org.w3c.dom.Node.TEXT_NODE)
        {
            NodeList children = node.getChildNodes();
            System.out.println(prefix + "Children("+ children.getLength()+"):");
            if ( children.getLength() > 0)
            {
                System.out.print(prefix+ "    ");
                for (int i = 0; i < children.getLength(); i++)
                {
                    Node child = children.item(i);
                    System.out.print(" ["+i+"] " + child.getNodeName());
                }
                System.out.println();
            }

        }
    }
    // Display Entity Information
    static void displayEntityInfo(String prefix, Entity entity)
    {
        System.out.println(prefix + "Entity information");
        System.out.println(prefix + "     public id:"+ entity.getPublicId());
        System.out.println(prefix + "     system id:"+ entity.getSystemId());
        System.out.println(prefix + "     notation name:"+ entity.getNotationName());
        displayNodeInfo(prefix,entity);

        NodeList children = entity.getChildNodes();
        if ( children.getLength() == 0)
            System.out.println(prefix + "     Has 0 children");
        else
            System.out.println(prefix + "     Children(" + children.getLength()+ ")");

        for (int i = 0; i < children.getLength(); i++)
        {
            Node child = (Entity)children.item(i);
            System.out.println("    child(" + i + ")");
            displayNodeInfo("        ",child);
        }

    }
    // Display Document information
    static void displayDocumentInfo(Document document)
    {
        DocumentType docTypeInfo = document.getDoctype();
        System.out.println(" ");
        System.out.println("Document Type Information");
        System.out.println("----------------");
        System.out.println("name:"+document.getNodeName());
        System.out.println("type:("+document.getNodeType()+ ")" +mapNodeTypeToString(document.getNodeType()));
        System.out.println("value:"+ document.getNodeValue());
        System.out.println("    Properties");
        System.out.println("    Name Property:"+docTypeInfo.getName());
        NamedNodeMap entities = docTypeInfo.getEntities();
        System.out.println("    contains " + entities.getLength() + " entities");
        NamedNodeMap notations = docTypeInfo.getNotations();
        if ( notations != null)
            System.out.println("    contains " + notations.getLength() + " notations");
        else
            System.out.println("    contains (null) notations");

        if ( entities != null && entities.getLength() > 0)
        {
            System.out.println("    Entities");
            for (int i = 0; i < entities.getLength(); i++)
            {
                //
                // Note that in the SUN implementation this works as expected
                // The IBM implementation causes a class cast exception.
                try
                {
                    Entity entity = (Entity)entities.item(i);
                    System.out.println("    Entity(" + i + ")");
                    displayEntityInfo("        ",entity);
                }
                catch (Exception e) { System.out.println("exception! " + e); } ;
            }
        }

        if ( notations != null && notations.getLength() > 0)
        {
            System.out.println("    Notations");
            for (int i = 0; i < notations.getLength(); i++)
            {
                Node node = notations.item(i);
                System.out.println("    Notation(" + i + ")");
                displayNodeInfo("        ",node);
            }
        }


    }

    // Process all the children of a node.
    public static void displayChildren(String prefix,Node parent)
    {
        NodeList children = parent.getChildNodes();
        if ( children == null || children.getLength() == 0) return;
        for (int i = 0; i < children.getLength(); i++)
        {
            try
            {
                Node node = children.item(i);
                displayNodeInfo(prefix,node);
                displayChildren(prefix + "    ",node);
            }
            catch (Exception e)
            {
            }
         }

    }

    public static Document openDocument (String xmlDocumentPath) { return openDocument(xmlDocumentPath,true);} ;
    public static Document openDocument (String xmlDocumentPath, boolean validate)
    {

        Document document;
        try
        {
            document = XmlDocument.createXmlDocument(new FileInputStream(xmlDocumentPath),validate);
            return document;

        }
        catch (Exception e)
        {
            return null;
        }
   }

}
