/* DomMenu.java */
package sams.chp7;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import com.sun.xml.parser.Resolver;
import com.sun.xml.tree.XmlDocument;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.w3c.dom.*;

public class DomMenu extends JFrame implements ActionListener
{
    XmlDocument doc;
    static final String MENU_NAME_ATTRIBUTE="name";
    static final String MENU_ACTION_ATTRIBUTE="action";
    static final String MENU_ITEM_NAME = "MENU-ITEM";
    static final String MENU_NAME = "MENU";

    public Insets getInsets()
    {
        return new Insets(25,5,5,5);
    }

    public static JMenuBar menuFactory(Document menuMLDocument, ActionListener al)
    {
        JMenuBar mbar = new JMenuBar();

        Element root = menuMLDocument.getDocumentElement();
        // process top-level menus
        NodeList menus = root.getChildNodes();
        int numMenus = menus.getLength();
        for (int i=0; i < numMenus; i++)
        {
            Node menuNode = menus.item(i);
            NamedNodeMap attrs = menuNode.getAttributes();
            Node name = attrs.getNamedItem(MENU_NAME_ATTRIBUTE);
            JMenu menu = new JMenu(name.getNodeValue());
            mbar.add(menu);
            // recursively process this menu
            processMenu(menuNode, menu, al);
        }

        return mbar;
    }

    private static void processMenu(Node menuNode, JMenu menu, ActionListener al)
    {
        NodeList children = menuNode.getChildNodes();
        int len = children.getLength();
        for (int i=0; i < len; i++)
        {
            Node n = children.item(i);
            if (n.getNodeName().equals(MENU_ITEM_NAME))
            {
                NamedNodeMap attrs = n.getAttributes();
                Node action = attrs.getNamedItem(MENU_ACTION_ATTRIBUTE);
                Node name = attrs.getNamedItem(MENU_NAME_ATTRIBUTE);
                JMenuItem item = new JMenuItem(name.getNodeValue());
                item.addActionListener(al);
                if (action != null)
                    item.setActionCommand(action.getNodeValue());
                menu.add(item);
            }
            else if (n.getNodeName().equals(MENU_NAME))
            {
                NamedNodeMap attrs = n.getAttributes();
                Node name = attrs.getNamedItem(MENU_NAME_ATTRIBUTE);
                JMenu submenu = new JMenu(name.getNodeValue());
                menu.add(submenu);
                processMenu(n, submenu, al);
            }
        }
    }

    public void actionPerformed(ActionEvent evt)
    {
        String cmd = evt.getActionCommand();
        System.out.println("Action: " + cmd);
        if (cmd.equals("Quit"))
            System.exit(0);
    }

    public DomMenu(String fileName) throws Exception
    {
        super(fileName);

        // create a SAX InputSource
        InputSource is = Resolver.createInputSource(new File(fileName));

        // create a DOM Document
        doc = XmlDocument.createXmlDocument(is, true);

        JMenuBar mbar = menuFactory(doc, this);
        setJMenuBar(mbar);

        addWindowListener(new WindowAdapter()
                          {
                            public void windowClosing(WindowEvent we)
                            { System.exit(1); }
                          });

        setLocation(100,100);
        setSize(600,400);
        setVisible(true);
    }

    public static void main(String args[])
    {
        if (args.length < 1)
        {
            System.out.println("USAGE: java sams.chp9.DomMenu xmlfile");
            System.exit(1);
        }

        try
        {
            new DomMenu(args[0]);
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}
