/* DomTable.java */
package sams.chp7;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import com.sun.xml.parser.Resolver;
import com.sun.xml.tree.XmlDocument;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.w3c.dom.*;

public class DomTable extends JFrame
{
    XmlDocument doc;

    public Insets getInsets()
    {
        return new Insets(25,5,5,5);
    }

    class DomTableModel extends AbstractTableModel
    {
        Document doc;
        ArrayList listeners = new ArrayList();
        NodeList records, columns;

        public DomTableModel(Document doc, String recordTagName)
        {
            this.doc = doc;
            records = doc.getDocumentElement().getElementsByTagName(recordTagName);
            columns = records.item(0).getChildNodes();
        }

        // Table model methods
        public int getRowCount()
        {
            return records.getLength();
        }

        public int getColumnCount()
        {
            // subelements of this element
            return columns.getLength();
        }

        public String getColumnName(int column)
        {
            return columns.item(column).getNodeName();
        }

        public Object getValueAt(int row, int column)
        {
            Node n = records.item(row);
            NodeList nl = n.getChildNodes();
            Node n2 = nl.item(column);
            String val = n2.getNodeValue();
            if (val == null)
            {
                // Node is an element
                // get text node
                Node n3 = n2.getFirstChild();
                if (n3 != null)
                    val = n3.getNodeValue();
            }

            return val;
        }
    }


    public DomTable(String fileName, String recordTagName) throws Exception
    {
        super(recordTagName + " Table");

        // create a SAX InputSource
        InputSource is = Resolver.createInputSource(new File(fileName));

        // create a DOM Document
        doc = XmlDocument.createXmlDocument(is, true);

        // create a DomTableModel
        DomTableModel dtm = new DomTableModel(doc, recordTagName);

        // Create a Table
        JTable tbl = new JTable(dtm);

        // Create a scroll pane
        JScrollPane scroll = new JScrollPane(tbl);

        getContentPane().add("Center", scroll);

        addWindowListener(new WindowAdapter()
                          {
                            public void windowClosing(WindowEvent we)
                            { System.exit(1); }
                          });

        setLocation(100,100);
        setSize(600,400);
        setVisible(true);
    }

    public static void main(String args[])
    {
        if (args.length < 2)
        {
            System.out.println("USAGE: java sams.chp9.DomTable xmlfile recordTag");
            System.exit(1);
        }

        try
        {
            new DomTable(args[0], args[1]);
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}
