/* DomViewer.java */
package sams.chp7;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import com.sun.xml.parser.Resolver;
import com.sun.xml.tree.XmlDocument;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.w3c.dom.*;

public class DomViewer extends JFrame implements ActionListener
{
    XmlDocument doc;
    
    public Insets getInsets()
    {
        return new Insets(25,5,5,5);    
    }
    
    class DomTreeModel implements TreeModel
    {
        Document doc;
        ArrayList listeners = new ArrayList();
        
        public DomTreeModel(Document doc)
        {
            this.doc = doc;
        }
        
        // Tree model methods
        public Object getRoot()
        {
            return doc.getDocumentElement();
        }

        public Object getChild(Object parent, int i)
        {
            Object o = null;
            NodeList children = ((Node)parent).getChildNodes();
            return children.item(i);
        }

        public int getChildCount(Object parent)
        {
            NodeList children = ((Node)parent).getChildNodes();
            return children.getLength();
        }

        public int getIndexOfChild(Object parent, Object child)
        {
            NodeList children = ((Node)parent).getChildNodes();
            int size = children.getLength();
            int i = -1;
            for(i = 0; i<size; i++)
            {
                Node n = children.item(i);
                if (n == child)
                    break;
            }
            return i;
        }

        public boolean isLeaf(Object node)
        {
            boolean hasChildren = ((Node)node).hasChildNodes();
            return !hasChildren;
        }

        public void valueForPathChanged(TreePath path, Object newValue)
        {
            Node n = (Node) path.getLastPathComponent();
            n.setNodeValue((String)newValue);
        }
    
        public void addTreeModelListener(TreeModelListener l)
        {
            listeners.add(l);
        }

        public void removeTreeModelListener(TreeModelListener l)
        {
            listeners.remove(l);
        }
    }
    
    class DomTreeCellRenderer implements TreeCellRenderer
    {        
        public Component getTreeCellRendererComponent(JTree tree, 
                                                      Object value, 
                                                      boolean selected, 
                                                      boolean expanded, 
                                                      boolean leaf, 
                                                      int row, 
                                                      boolean hasFocus) 
        {
            Component c = null;
            Color highlight = new Color(102,255,255);

            //find and see if the node is element node
            if (!leaf) 
            {
                JLabel l = new JLabel();
                l.setOpaque(true);
                c = l;
                if ( selected ) 
                {
                    l.setBackground(highlight);
                    l.setForeground(Color.black);
                    l.setText(((Node)value).getNodeName());
                }
                else 
                {
                    l.setBackground(Color.white);
                    l.setForeground(new Color(0,51,204));
                    l.setText(((Node)value).getNodeName());
                }
            }
            else  // a leaf node
            {
                JTextField f = new JTextField(30);
                //f.setEditable(true);
                c = f;
                if ( selected ) 
                {
                    f.setBackground(new Color(195, 195, 250));
                    f.setForeground(Color.black);
                    f.setText(((Node)value).getNodeValue());
                }
                else 
                {
                    f.setBackground(Color.white);
                    f.setForeground(Color.black);
                    f.setText(((Node)value).getNodeValue());
                }
            }

            return c;
        }
    }

    class DomTreeCellEditor  extends DefaultCellEditor 
                             implements TreeCellEditor
    {
        private JTree tree;
        
        public DomTreeCellEditor(JTree tree)
        {
            super(new JTextField(30));
            this.tree = tree;
        }
        
        public boolean canEditImmediately(EventObject e)
        {
            return true;    
        }
        
        public boolean isCellEditable(EventObject e)
        {
            boolean result = false;
            
            if (e instanceof MouseEvent)
            {
                MouseEvent me = (MouseEvent) e;
                TreePath path = tree.getPathForLocation(me.getX(), me.getY());
                if (path != null)
                {
                    Node n = (Node) path.getLastPathComponent();
                    if (n != null && !n.hasChildNodes())
                        result = true;
                }
            }
            
            return result;
        }        
        
        public Component getTreeCellEditorComponent(JTree tree, 
                                                    Object value, 
                                                    boolean selected, 
                                                    boolean expanded, 
                                                    boolean leaf, 
                                                    int row) 
        {
            JTextField f = (JTextField) getComponent();
            Color highlight = new Color(102,255,255);
            if (leaf) 
            {
                if ( selected ) 
                {
                    f.setBackground(highlight);
                    f.setForeground(Color.black);
                    f.setText(((Node)value).getNodeValue());
                }
                else 
                {
                    f.setBackground(Color.white);
                    f.setForeground(Color.black);
                    f.setText(((Node)value).getNodeValue());
                }
            }
            else
                return null;

            return f;
        }
    }

    public DomViewer(String fileName, boolean defaultRenderer) throws Exception
    {
        super(fileName);
        
        // add a simple menu
        JMenuBar bar = new JMenuBar();
        setJMenuBar(bar);
        JMenu fileMenu = new JMenu("File");
        JMenuItem saveItem = new JMenuItem("Save");
        saveItem.addActionListener(this);
        JMenuItem quitItem = new JMenuItem("Quit");
        quitItem.addActionListener(this);
        fileMenu.add(saveItem);
        fileMenu.add(quitItem);
        bar.add(fileMenu);
        
        // create a SAX InputSource
        InputSource is = Resolver.createInputSource(new File(fileName));

        // create a DOM Document
        doc = XmlDocument.createXmlDocument(is, true);
        
        // Create a Tree Model from the Document
        DomTreeModel model = new DomTreeModel(doc);
        
        // Create a renderer
        DomTreeCellRenderer renderer = new DomTreeCellRenderer();
        
        // Create the JTree
        JTree tree = new JTree(model);
        tree.putClientProperty("JTree.lineStyle", "Angled");
        
        // Create an editor
        DomTreeCellEditor editor = new DomTreeCellEditor(tree);        
        
        if (!defaultRenderer)
        {
            tree.setCellRenderer(renderer);
            tree.setEditable(true);        
            tree.setCellEditor(editor);
        }
        
        // Create a scroll pane
        JScrollPane scroll = new JScrollPane(tree);
        
        getContentPane().add("Center", scroll);
        
        addWindowListener(new WindowAdapter()
                          {
                            public void windowClosing(WindowEvent we)
                            { System.exit(1); }
                          });
                          
        setLocation(100,100);
        setSize(600,400);
        setVisible(true);
    }

    public void actionPerformed(ActionEvent evt)
    {
        String command = evt.getActionCommand();
        if (command.equalsIgnoreCase("Quit"))
        {
            System.exit(0);    
        }
        else if (command.equalsIgnoreCase("Save"))
        {
            JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
            int stat = chooser.showSaveDialog(this);
            if (stat == JFileChooser.APPROVE_OPTION)
            {
                File f = chooser.getSelectedFile();
                try
                {
                    FileWriter fw = new FileWriter(f);
                    doc.write(fw);
                    fw.close();
                } catch (IOException ioe)
                  {
                    System.out.println("Error writing to: " + f);  
                  }                
            }
        }
    }

    public static void main(String args[])
    {
        if (args.length < 1)
        {
            System.out.println("USAGE: java sams.chp9.DomViewer xmlfile [-default]");
            System.exit(1);
        }
        
        try
        {
            boolean defaultRendering = false;
            if (args.length == 2 && args[1].equals("-default"))
                defaultRendering = true;
                
            new DomViewer(args[0], defaultRendering);
        } catch (Throwable t)
          {
            t.printStackTrace();  
          }
    }
}
