package sams.chp8;
import java.lang.reflect.*;
import java.beans.*;

//
// a simple class which dumps information about another class
//

public class DumpBean
{

	public static Object DumpObject(String className) throws InstantiationException,IllegalAccessException
	{
		Class theClass = null;
        try
		{
        	theClass = Class.forName(className);
        	System.out.println("Information about:" + theClass.getName());
			System.out.println("---\n");

			Package pkg  = theClass.getPackage();
        	System.out.println(" Package ");
        	if (null == pkg )
				System.out.println("    package:not part of a package(default)");
			else
				System.out.println("    package:" + pkg.toString());
			System.out.println("---\n");


			Class[] interfaces = theClass.getInterfaces();
        	System.out.println(" Interfaces(" + interfaces.length + ")");
        	for (int i = 0; i < interfaces.length; i++)
        	{
				Class anInterface = interfaces[i];
				System.out.println("    interface["+i+"]" + anInterface.toString());
			}
			System.out.println("---\n");

			Constructor[] ctors = theClass.getConstructors();
        	System.out.println(" Ctors(" + ctors.length + ")");
        	for (int i = 0; i < ctors.length; i++)
        	{
				Constructor ctor = ctors[i];
				System.out.println("    ctor["+i+"]" + ctor.toString());
			}
			System.out.println("---\n");

			Field[] fields = theClass.getDeclaredFields();
        	System.out.println(" Fields(" + fields.length + ")");
        	for (int i = 0; i < fields.length; i++)
        	{
				Field field = fields[i];
				int modifiers = field.getModifiers();
				System.out.println("    Field["+i+"]" + field.toString());
				if ( java.lang.reflect.Modifier.isTransient(modifiers)) System.out.println("    isTransient");
				if ( java.lang.reflect.Modifier.isProtected(modifiers)) System.out.println("    isProtected");
				if ( java.lang.reflect.Modifier.isStatic(modifiers)) System.out.println("    isStatic");
			}
			System.out.println("---\n");


        	Method[] methods = theClass.getDeclaredMethods();

        	System.out.println(" Methods(" + methods.length + ")");
        	for (int i = 0; i < methods.length; i++)
        	{
				Class returnType = methods[i].getReturnType();
				Class[] parameterTypes = methods[i].getParameterTypes();
				System.out.println("    Method["+i+"]" + methods[i].getName());
				System.out.println("    " + methods[i].toString());
				System.out.println("      returns:"+ returnType.toString());
				System.out.println("      takes "+parameterTypes.length + " parameters");
				for (int j =0; j < parameterTypes.length;j++)
				{
					System.out.println("       Parameter[" + j + "]:" + parameterTypes[j].toString()
					+ (parameterTypes[j].isPrimitive()?" isPrimitive":" not Primitive")
					);
				}
			}

			System.out.println("\n\n");

        }
        catch (SecurityException e)
        {
			System.out.println(e);
		}
        catch (ClassNotFoundException e)
        {
			System.out.println("Class:"+className + " not found!");
			System.out.println(e);
		}

		return theClass!=null? theClass.newInstance():null;

	}

	public static void main (String argv [])
    {
        if (argv.length == 0)
        {

            System.err.println("Usage: java sams.chp10.DumpBean someClass" );
            System.exit(1);
        }

		try
		{
        	Object bean = DumpObject(argv[0]);
			if ( null == bean)
			{
				System.out.println("Failed to open class from " + argv[0]);
				System.exit(1);
			}
		}
		catch (Exception e)
		{
			System.out.println(e);

		}
	}
}