package sams.chp8;
import java.lang.reflect.*;
import java.beans.*;
import java.io.*;
import sams.chp8.*; // For XMLUtil.openDocument
import org.w3c.dom.*;


public class XMLBean
{

	//
	// Convinience method to create a new Object which
	// can be used as an argument to the setter method
	//
	static Object buildSimpleArgument(Class type, String value)
	{
		Object argument = null;

		if (type == char.class)
		{
			char c = (char) (Integer.decode(value).intValue());
			argument = new Character(c);
		}
		else if ( type == java.lang.String.class)
		{
			argument = value;
		}
		else if (type == boolean.class)
			argument = new Boolean(value);
		else if (type == byte.class)
		 	argument = new Byte(value);
		else if (type == int.class)
			argument = new Integer(value);
		else if (type == long.class)
			argument = new Long(value);
		else if (type == short.class)
			argument = new Short(value);
		else if (type == float.class)
			argument = new Float(value);
		else if (type == double.class)
			argument = new Double(value);
		else
			return null;
	   return argument;
	}


	//
	// load a property
	// This method does the bulk of the work of loading a JavaBean property
	//
	static void loadProperty(Node aProperty,Object object)
				throws NoSuchMethodException, IllFormedBeanException
	{
		if ( null == aProperty)
		{
			return; // something strange happened!
		}


		NamedNodeMap attributes = aProperty.getAttributes();
        if ( null == attributes || attributes.getLength() == 0)
        {
			throw new IllFormedBeanException("Cannot Name attributes for "
												 + aProperty.getNodeName()
												 + " in class "
												 + object.getClass().getName());
        }
		String propertyName = null;
		for ( int i = 0; i < attributes.getLength();i++ )
		{
        	Node attribute = attributes.item(i);
        	if (attribute.getNodeName().equals("Name"))
			{
				propertyName = attribute.getNodeValue();
				break;
			}
		}
		if ( null == propertyName)
		{
			throw new IllFormedBeanException("Cannot find Name attribute for "
												 + propertyName + " in class " +
												 object.getClass().getName());
		}

		//
		// get the class and find the method which has the same name
		//
		// We use property descriptors to find setter method
		// and then use that method to store the value into the object
		// We could use a hashmap to store all the desciptors and then access them
		// by hash but this method, while slower, show more what is going on with
		// the property descriptors.
		try
		{
			Class    theClass = object.getClass();
			PropertyDescriptor propDescs[] = Introspector.getBeanInfo(theClass).getPropertyDescriptors();
			PropertyDescriptor pd = null;
			Method setter = null;
			Class argumentType = null;

			for (int i = 0; setter == null && i < propDescs.length; i++)
			{
				if (propDescs[i].getName().equals(propertyName))
				{
					pd = propDescs[i];
					setter = pd.getWriteMethod();
					argumentType = pd.getPropertyType();
				}
			}
			if ( null == setter )
			{
				throw new IllFormedBeanException("Cannot find setter method for "
												 + propertyName + " in class " +
												 theClass.getName());
			}
			else if ( null == argumentType  )
			{
				throw new IllFormedBeanException("Cannot determine argument type for "
												 + propertyName + " in class " +
												 theClass.getName());
			}
			//
			// Ok, one more step before we set the value.
			// We need to get the value of the property as
			// an argument that can be passed to the setter method
			// there are two possiblities here:
			//	b: The object is a primitive type or string
			//     use buildSimpleArgument to create the
			//  b: the object is an embedded bean
			//     instantiate it and use the result call the setter.
			//

			Object[] args = { null };
			NodeList propertyChildren = aProperty.getChildNodes();

			//
			// Is the argument a primitive type?
			// That is long, float, doublel etc?
			// Then look for the text node that represents it
			// and use buildSimpleArgument to construct an appropriate
			// object to represent that value
			//
			if (argumentType.isPrimitive() || argumentType == java.lang.String.class)
			{
				String value = null;
				for (int i = 0; i < propertyChildren.getLength(); i++)
				{
					Node aNode = propertyChildren.item(i);
					if (aNode instanceof Text)
					{
						value = aNode.getNodeValue();
						break;
					}
				}
				if ( null == value)
				{
					throw new IllFormedBeanException("Value not found in XML Bean for"
													 + propertyName + " in class " +
													 theClass.getName());
				}
				args[0] = buildSimpleArgument(argumentType, value);
			}
			//
			// If its not a primitive type
			//
			else
			{
				//
				// look for a child that says its a javabean
				//
				for (int i = 0; i < propertyChildren.getLength(); i++)
				{
					Node aNode = propertyChildren.item(i);
					if ( aNode.getNodeName().equals("JavaBean") )
					{
						args[0] = newBeanInstance(aNode);
					}
				}
			}

			//
			// At this point if args[0] is still null we were not able to build the argument list
			// throw an exception
			//
			if (null == args[0])
			{
				throw new IllFormedBeanException("Cannot determine parameters to set method for "
												  + propertyName + " in class "
												  + theClass.getName());
			}

			//
			// We found the setter method all we need to do is call it.
			//
			setter.invoke(object, args );
		}
		catch (Exception e)
		{
			return;
		}
	}

	//
	// Fill the bean based on the field names
	//
	static void loadBeanProperties(Object object,Node bean)
	{
		// get all the children of the bean
		// the should all be Properties children
		// later we will examine what we should
		// do if they are JavaBeans (beans with in beans!)
		NodeList propertiesNL = bean.getChildNodes();
		for (int i = 0; i < propertiesNL.getLength(); i++)
		{
			Node propertiesNode = propertiesNL.item(i);
			if ( propertiesNode.getNodeName().equals("Properties"))
			{
				DOMArray propertiesValues = new DOMArray(propertiesNode,"Property");
				for ( int j = 0; j < propertiesValues.size(); j++)
				{
					try
					{
						loadProperty(propertiesValues.getAsNode(j),object);
					}
					catch (NoSuchMethodException e)
					{
						System.out.println("Unexpected NoSuchMethodException loading a property");

					}
					catch(IllFormedBeanException ifbe)
					{
						System.out.println("Unexpected IllFormedBeanException:" + ifbe.getMessage());
					}
				}
			}
		}
	}


	//
	// Create a JavaBean from a string representing a uri to a XML file
	// containing the JavaBean description
	//
	public static Object newBeanInstance(String xmlJavaBean)
				         throws InstantiationException,IllegalAccessException, ClassNotFoundException
	{
		Document document = XMLUtil.openDocument(xmlJavaBean,false); // create a non-validating parser
		if ( null == document)
			return null;  // Open failed cant create new instance.
		return newBeanInstance(document);
	}

	//
	// Create a bean from a document by finding the first bean instance
	// and returning a beans based on it.
	//
	public static Object newBeanInstance(Document document)
				         throws InstantiationException,IllegalAccessException, ClassNotFoundException
	{
		//
		// Find the first JavaBean node
		//
		NodeList beans = document.getElementsByTagName( "JavaBean" );
		if ( beans.getLength() < 1)
		{
			return null;
		}
		return newBeanInstance(beans.item(0));
	}

	//
	// Create a JavaBean starting from the DOM Tree node what represents the bean
	// That is the <JavaBean Name=... element
	//
	public static Object newBeanInstance(Node aBean)
				         throws InstantiationException,IllegalAccessException, ClassNotFoundException
	{
		Class theClass = null;
		NamedNodeMap attributes = aBean.getAttributes();
        if ( null == attributes || attributes.getLength() == 0)
        {
			return null;
        }
		String beanName = null;
		for ( int i = 0; i < attributes.getLength();i++ )
		{
        	Node attribute = attributes.item(i);
        	if (attribute.getNodeName().equals("ClassName"))
			{
				beanName = attribute.getNodeValue();
				break;
			}
		}
		if( null == beanName )
		{
			System.out.println("Pretty weird Found a bean with no name!");
			return null;
		}

		//
		// load the class
		//
        theClass = Class.forName(beanName);
		if ( null == theClass)
		{
			// most likely class not found
			return null;
		}
		//
		// If we are here we found the class
		// create a newInstance of the class
		// get all the properties and start setting them one by one.
		//
		Object  object = theClass.newInstance();
		loadBeanProperties(object,aBean);

		return object;
	}


	//
	// The bulk of writing a bean is done in output properties
	// We first find all the fields, skipping those which are transient
	// and get the associated property descriptor.
	// We then use that property descriptor to get the reader method
	// and then write the value.  We use each primitive types
	// toString method to get a printable representation of that object.
	// For non-primitive objects we simply call outputBean with the
	// object and recurse thru the beans within beans!
	//
	static boolean outputProperties(int level,PrintStream ps, Object bean, Class theClass)
	{

		// go to the fields directly
		Field[] fields = theClass.getDeclaredFields();
		for (int i = 0; i < fields.length; i++)
		{
			Field field = fields[i];
			Method getter = null;
			//
			// Skip transient fields
			//
			int modifiers = theClass.getModifiers();
			if ( modifiers == java.lang.reflect.Modifier.TRANSIENT )
			{
				continue;
			}
			Class type = null;
			String fieldName = field.getName();
			try
			{
				PropertyDescriptor propDescs[] = Introspector.getBeanInfo(theClass).getPropertyDescriptors();
				PropertyDescriptor pd = null;
				// find the property descriptor and getter for this field
				for (int j = 0; pd == null && j < propDescs.length; j++)
				{
					if (propDescs[j].getName().equals(fieldName))
					{
						pd = propDescs[j];
						getter = pd.getReadMethod();
						type = pd.getPropertyType();
					}
				}

			} catch ( IntrospectionException ie) {}
			if ( null == getter)
			{
				System.out.println("Cannot find getter for " + field.getName());
				continue;  // no getter skip field
			}
			Object value = null;
			try {
				value = getter.invoke(bean,null);
			} catch (Exception e) { value = null; };

			if ( null == value ) continue;  // Couldn't get value skip it
			//
			// is it primitive or a string?
			//
			if ( type.isPrimitive() ||
				 type == java.lang.String.class)
			{
				//
				// just write it
				//
				prefix(ps,level);ps.print("<Property Name=\""+fieldName + "\">");
				try
				{
					Method toString = value.getClass().getMethod("toString", null);
					String valueAsString = (String)toString.invoke(value, null);
					ps.print(valueAsString);
				} catch (Exception e) { System.out.println("toString invoke failed" + e);};
				ps.println("</Property>");
			}
			else
			{
				prefix(ps,level);ps.println("<Property Name=\""+fieldName + "\">");
				outputBean(level+1,ps,value);
				prefix(ps,level);ps.println("</Property>");
			}
		}

		return true;

	}

	//
	// Output bean is the start of writing a bean object
	// It prints the appropriate class info and then calls outputProperties
	// to write the underlying properties.
	//
	static boolean outputBean(int level, PrintStream ps, Object bean)
	{
		Class theClass = bean.getClass();
		prefix(ps,level);ps.println("<JavaBean ClassName=\""+theClass.getName() + "\">");
		prefix(ps,level+1);ps.println("<Properties>");

		boolean result = outputProperties(level+2,ps,bean, theClass);

		prefix(ps,level+1);ps.println("</Properties>");
		prefix(ps,level);ps.println("</JavaBean>");
		return result;
	}

	static void prefix(PrintStream ps,int level)
	{
		if ( level == 0) return;
		for ( int i = 0; i < level; i++) ps.print("    ");
	}

	static void XMLBeanDTD(PrintStream ps)
	{
		ps.println("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
		ps.println("<!DOCTYPE JavaBean [");
		ps.println("<!ELEMENT JavaBean (Properties)>");
		ps.println("<!ATTLIST JavaBean ClassName CDATA #REQUIRED>");
		ps.println("<!ELEMENT Properties (Property*)>");
		ps.println("<!ELEMENT Property (#PCDATA)>");
		ps.println("<!ATTLIST Property Name CDATA #REQUIRED >");
		ps.println("]>\n");

	}
	public static boolean storeBeanInstance(PrintStream ps, Object bean)
	{
		XMLBeanDTD(ps);
		return outputBean(0,ps,bean);
	}
	public static boolean storeBeanInstance(String path, Object bean)
	{
		//
		// Open a stream to the file
		//
		boolean result = false;
		try
		{
			PrintStream pOut;
			pOut = new PrintStream (new FileOutputStream(path));
			result = storeBeanInstance(pOut,bean);
		}catch (IOException e){ }

		return result;
	}

}