package sams.chp8.bmp;

import java.io.Serializable;
import java.rmi.RemoteException;
import javax.ejb.*;
import java.util.*;
import java.lang.reflect.*;
import java.beans.*;


public class AlarmClockBean implements EntityBean {
    public int                          id;

    transient protected EntityContext   context;

    public String                       station;
    public String                       wakeUpTime;
    public String                       isFm;
	final static int					debug =0;
	final static int					error =1;

	void D(String msg)
	{
		System.out.println( msg );
	}
	void E(String msg)
	{
		System.err.println( msg );
	}


	protected String generatePrimaryKey() throws CreateException
	{
		Method[] methods = this.getClass().getDeclaredMethods();
		Class pkClass = null;
		for (int i = 0; i < methods.length; i++)
		{
			// EJB create is required to return an instance of the primary key
			if ( methods[i].getName().equals("ejbCreate"))
			{
				pkClass = methods[i].getReturnType();
			}
		}

		if ( null == pkClass )
		{
			throw new CreateException("Cannot determine Primary Key fields");
		}

		D("Found primary key class" + pkClass.getName());

		//
		// walk the primaryKey.
		// get each field, then get the fields value from the object not
		// the primary key
		//
		String stringKey="";
		Field[] fields = pkClass.getDeclaredFields();
		Field[] fieldsBean = this.getClass().getDeclaredFields();
		if ( null == fields )
			D("No fields found for "+ this.getClass().getName() + "!");
		else
		{
			D("Found " + fields.length + " fields for " + this.getClass().getName() + "!");
			for ( int i=0; i < fields.length; i++)
			{
				try {
					D("Attempting get on field["+i +"]" + fields[i].getName());
					// could handle non primitive values recursively...
					//
					// find the associated field in bean
					//
					Field beanEqField =null;
					for (int j = 0; beanEqField == null && j <fieldsBean.length; j++)
					{
						if ( fieldsBean[j].getName().equals( fields[i].getName() ) )
						{
							// we could do a lot more testing for type etc here.
							beanEqField = fieldsBean[i];
						}
					}
					if ( null == beanEqField )
					{
						D("Illformed bean!  pk field not found in bean!");
						continue;
					}
					else
						D("Found bean equiv field " + beanEqField.toString());
					Object value = beanEqField.get(this);
					if ( null == value)
					{
						D("get for field["+i+"] return null!");
						continue;
					}
					else
						D("get for field["+i+"] returned " + value);
					//if ( value.getClass().isPrimitive() != true)
					//{
					//	D("skipping non-primitive field " + value.getClass().toString());
					//	continue;
					//}
					int modifiers = value.getClass().getModifiers();
					if ( java.lang.reflect.Modifier.isTransient(modifiers))
					{
						D("Skipping transient field" + value.getClass().getName());
						continue;
					}
					if ( java.lang.reflect.Modifier.isStatic(modifiers))
					{
						D("Skipping static field" + value.getClass().getName());
						continue;
					}

					D("Attempting getMethod for toString method");
					Method toString = value.getClass().getMethod("toString", null);
					if ( null != toString)
					{
						String valueAsString = (String)toString.invoke(value, null);
						D("String value = " + valueAsString);
						String tmp = stringKey + valueAsString;
						stringKey = tmp;
					}
					else
						D("toString method not found for " + value.getClass().getName());

				}
				catch (NullPointerException npe){ E(npe.getMessage()); }
				catch (InvocationTargetException ive) {E(ive.getMessage()); }
				catch (NoSuchMethodException nsme) {E(nsme.getMessage());}
				catch (IllegalArgumentException iare) {E(iare.getMessage());}
				catch (IllegalAccessException iae){E(iae.getMessage());}
			}
		}
		//
		// got enough to make a key
		//
		D("String for key = " + stringKey);
		return stringKey;
	}

    public void ejbActivate() {}
    public AlarmClockPK ejbCreate() throws CreateException {

        // Generate a unique ID for this EJB.
        // Most EJBs pass this value in.

		D("Entered ejbCreate");

        this.id = (int) (Math.random() * 100000);

		generatePrimaryKey();

		AlarmClockPK key = new AlarmClockPK();
		key.id = this.id;
		return key;
    }
    public AlarmClockPK ejbCreate(String time, String station,
                                     boolean isFm) throws CreateException {

        // Generate a unique ID for this EJB.
        // Most EJBs pass this value in.
        this.id = (int) (Math.random() * 100000);

	    this.station = station;
	    this.isFm = new Boolean(isFm).toString();

		generatePrimaryKey();

		AlarmClockPK key = new AlarmClockPK();
		key.id = this.id;
		return key;
    }
    public AlarmClockPK ejbFindByPrimaryKey(AlarmClockPK pk)
        throws FinderException, RemoteException {
        if (pk == null)
            throw new FinderException("primary key cannot be null");

        refresh(pk);
        return pk;
    }
    public void ejbLoad() throws RemoteException {
            refresh((AlarmClockPK) context.getPrimaryKey());
    }
    public void ejbPassivate() throws RemoteException {}
    public void ejbPostCreate() {}
    public void ejbPostCreate(String time, String station, boolean isFm) {}
    public void ejbRemove() throws RemoteException {

        // we need to get the primary key from the context because
        // it is possible to do a remove right after a find, and
        // ejbLoad may not have been called.

    }
    public void ejbStore() throws RemoteException {

    }
    public void setEntityContext(EntityContext ctx) {
        this.context = ctx;
    }
    public void unsetEntityContext() throws RemoteException {
        this.context = null;


	//
	// business methods
	//
    public String getAlarmTime() {
        return this.wakeUpTime;
    }
    public String getRadioStation() {
        return this.station;
    }
    public boolean isFmSet() {
        return new Boolean(this.isFm).booleanValue();
    }
    private void refresh(AlarmClockPK pk)
    {
    }
    public void setAlarmTime(String d) {
        this.wakeUpTime = d;
    }
    public void setFm(boolean fm) {
        this.isFm = new Boolean(fm).toString();
    }
    public void setRadioStation(String station) {
        this.station = station;
    }
    }
}





