/* AphorismsToHtmlTbl.java */
package sams.chp9;

import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.*;

import com.sun.xml.parser.Resolver;
import com.sun.xml.tree.XmlDocument;
import org.xml.sax.*;
import org.w3c.dom.*;

import com.jclark.xsl.sax.*;
import com.jclark.xsl.dom.*;

public class AphorismsToHtmlTbl extends HttpServlet 
                                implements DocumentHandler, ErrorHandler                
{
    static final String inputfile = "Aphorisms.xml";
    static final String sheetfile = "aphorismtbl.xsl";
    
    PrintWriter pw;

    /** Locator reference. */
    Locator loc;
    
    public void service(HttpServletRequest req, HttpServletResponse res)     
    {
        try
        {
            res.setContentType("text/html");
            pw = res.getWriter();
            
            File in = new File(inputfile);
            if (!in.exists())
            {
                pw.println("<HTML> <HEAD> <TITLE> Aphorisms TABLE </TITLE>" +
                            "</HEAD> <BODY> <H1> No Aphorisms entered. </H1> " +
                            "</BODY> </HTML>"); 
                return;
            }

            File sheet = new File(sheetfile);
            if (!sheet.exists())
            {
                pw.println("<HTML> <HEAD> <TITLE> Aphorisms TABLE </TITLE>" +
                            "</HEAD> <BODY> <H1> Error: No XSL Stylesheet. </H1> " +
                            "</BODY> </HTML>"); 
                return;                
            }
            // create a SAX InputSources
            InputSource is = Resolver.createInputSource(in);
            InputSource sis = Resolver.createInputSource(sheet);
            
            // create a Sun DOM Xml Processor impl
            SunXMLProcessorImpl sunImpl = new SunXMLProcessorImpl();
            
            // create an XT XSL processor 
            XSLProcessorImpl xslProcessor = new XSLProcessorImpl();
            
            // set the Sun impl to use a DOM as input
            xslProcessor.setParser(sunImpl);
            
            // load the stylesheet
            xslProcessor.loadStylesheet(sis);
            
            // set Document Handler
            xslProcessor.setDocumentHandler(this);
            
            // set Error Handler
            xslProcessor.setErrorHandler(this);
            
            // parse input
            xslProcessor.parse(is);       
        } catch (Exception e)
          {
            e.printStackTrace();
            try
            {
                res.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);  
            } catch (IOException ioe) { }
          }
    }
    
    /** method of the DocumentHandler Interface. */
    public void characters(char[] ch, int start, int length)
    {
        // Receive notification of character data inside an element.
        pw.print(new String(ch,start,length));
    }

    /** method of the DocumentHandler Interface. */
    public void endDocument()
    {
        // Receive notification of the end of the document.        
        pw.println("</BODY> </HTML>");
    }

    /** method of the DocumentHandler Interface. */
    public void endElement(java.lang.String name)
    {
        // Receive notification of the end of an element.
        pw.println("</" + name + ">");
    }

    /** method of the DocumentHandler Interface. */
    public void ignorableWhitespace(char[] ch, int start, int length)
    {
        // Receive notification of ignorable whitespace in element content.
    }

    /** method of the DocumentHandler Interface. */
    public void processingInstruction(java.lang.String target,
                               java.lang.String data)
    {
        // Receive notification of a processing instruction.
    }

    /** method of the DocumentHandler Interface. */
    public void setDocumentLocator(Locator locator)
    {
        // Receive a Locator object for document events.
        loc = locator;
    }

    /** method of the DocumentHandler Interface. */
    public void startDocument()
    {
        // Receive notification of the beginning of the document.
        pw.println("<HTML> <HEAD> <TITLE> Aphorisms TABLE </TITLE> </HEAD> <BODY>"); 
    }

    /** method of the DocumentHandler Interface. */
    public void startElement(java.lang.String name, AttributeList attributes)
    {
        // Receive notification of the start of an element.
        pw.print("<" + name + " ");
        for (int i = 0; i < attributes.getLength(); i++)
        {
            String attName = attributes.getName(i);
            String type = attributes.getType(i);
            String value = attributes.getValue(i);
            pw.print(attName + "=" + value + " ");
        }
        pw.println(">");
    }

    /** method of the ErrorHandler Interface. */
    public void error(SAXParseException e)
    {
        // Receive notification of a recoverable parser error.
        pw.println("Error Parsing XML! Reason:" + e + "<BR>");        
        pw.println("In " + loc.getSystemId() + ",at line " +
                    loc.getLineNumber() + " and col " +
                    loc.getColumnNumber() + "<BR>");
    }

    /** method of the ErrorHandler Interface. */
    public void fatalError(SAXParseException e)
    {
        // Report a fatal XML parsing error.
        pw.println("FATAL Error Parsing XML! Reason:" + e + "<BR>");        
        pw.println("In " + loc.getSystemId() + ",at line " +
                    loc.getLineNumber() + " and col " +
                    loc.getColumnNumber() + "<BR>");
    }

    /** method of the ErrorHandler Interface. */
    public void warning(SAXParseException e)
    {
        // Receive notification of a parser warning.
        pw.println("Warning Parsing XML! Reason:" + e + "<BR>");        
        pw.println("In " + loc.getSystemId() + ",at line " +
                    loc.getLineNumber() + " and col " +
                    loc.getColumnNumber() + "<BR>");
    }    
}
